#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h> 
#include <stdio.h>
#include "slm.h"

#define TEST "test.slm"
#define IMAGE	"test_image/long_bridge.slm"
#define SLM "/dev/slm"
#define TIME	0x00000fff
#define 	QUIT		0
#define 	POWER_ON	1
#define 	POWER_OFF	2
#define 	ENABLE_WRITE	3
#define 	DISABLE_WRITE	4
#define 	UPLOAD_FRAME	5
#define 	RUN		6
#define 	STOP		7
#define		DISPLAY		8
#define		PERIOD		9
#define 	IRL		10
#define 	HELP		11

#define NFRAMES 1024

void h_err(int);
#define ROW_SIZE (512)
#define IMAGE_LENGTH 512
#define SIZE 512*IMAGE_LENGTH
#define N 100000
main(){
int* dummy=NULL;
FILE	*fi,*fo;
int	dev;
unsigned char image[SIZE];
int pos,y,i,option,offset,frame,period=TIME;
int n=0,err;
char temp[NFRAMES];
struct BNS_irl_object	irl[NFRAMES];

	printf("Matt's test slm driver program\n");
	printf("Opening slm \"%s\"\n",SLM);
	dev=open(SLM,O_RDWR);
	if(dev<0){
		printf("Failed to open %s\n",SLM);
		perror("slm_test:FTO:");
		exit(1);
		}
	printf("periods to %x\n",TIME);
	h_err(ioctl(dev,BNS_SET_PERIODS,TIME));
	do{
		printf("Choose an option (help=%d):\n",HELP);
		scanf("%d",&option);
		switch(option){
			case HELP:
				printf("Power on	=%d\n",POWER_ON);
				printf("Power off	=%d\n",POWER_OFF);
				printf("Enable write	=%d\n",ENABLE_WRITE);
				printf("Disable write /\n Allow image	=%d\n",DISABLE_WRITE);
				printf("Upload frames	=%d\n",UPLOAD_FRAME);
				printf("Run		=%d\n",RUN);
				printf("Stop		=%d\n",STOP);
				printf("Display 1 frame	=%d\n",DISPLAY);
				printf("Change period	=%d\n",PERIOD);
				printf("IRL		=%d\n",IRL);
				printf("Quit		=%d\n",QUIT);
				break;
			case POWER_ON:
				printf("Setting slm power on\n");
				h_err(ioctl(dev,BNS_POWER_ON,dummy));
				break;
			case POWER_OFF:
				printf("Setting slm power off\n");
				h_err(ioctl(dev,BNS_POWER_OFF,dummy));
				break;
			case ENABLE_WRITE:
				printf("Enabling write\n");
				h_err(ioctl(dev,BNS_ENABLE_WRITE,dummy));
				break;
			case DISABLE_WRITE:
				printf("Disabling write\n");
				h_err(ioctl(dev,BNS_DISABLE_WRITE,dummy));
				break;
			case UPLOAD_FRAME:
				printf("Enter image name for upload:");
				scanf("%s",temp);
				printf("\nEnter image number to upload:");
				scanf("%d",&n);
				fi=fopen(temp,"r");
				if(fi>0){
					printf("\nReading %s\n",temp);
					fread(image,sizeof(unsigned char),SIZE,fi);	
					fclose(fi);
					printf("Uploading image to %d \n",n);
					h_err(ioctl(dev,BNS_UPLOAD_IMAGE,n));
					err=ROW_SIZE;
					for(pos=offset,y=0;y<512;y++,pos+=IMAGE_LENGTH){ 
						err=write(dev,image+pos,ROW_SIZE);
						if(err!=ROW_SIZE){
							printf("WRITE ERROR (writes enabled?)\n",err);
							break;
							}
						}
					printf("Done\n");
					}
				else{
					printf("\nCould not open file \"%s\"\n",temp);
					}

				break;
			case IRL:
				break;
				printf("Computing the IRL\n");
				for(i=0;i<NFRAMES;i++){
					irl[i].image=i;
					irl[i].periods=period;
					}
				printf("Setting the # irl images to %d\n",NFRAMES);
				h_err(ioctl(dev,BNS_SET_LIST_LENGTH,NFRAMES));
				printf("Uploading the frame list\n");
				h_err(ioctl(dev,BNS_SET_LIST,irl));
				printf("done, type %d to enable run\n",RUN);
				break;
			case RUN:
				break;
				printf("Enabling run, type %d to stop\n",STOP);
				h_err(ioctl(dev,BNS_START_LIST,BNS_START));
				break;
			case STOP:
				break;
				printf("Enabling run, type %d to stop\n",STOP);
				h_err(ioctl(dev,BNS_STOP_LIST,dummy));
				break;
			case DISPLAY:
				printf("Enter a frame to display: ");
				scanf("%d",&frame);
				h_err(ioctl(dev,BNS_DISPLAY_IMAGE,frame));
				break;
			case PERIOD:
				printf("Enter period length: ");
				scanf("%x",&period);
				printf("\nSetting time to %f seconds (%d)\n",(164.4e-6*(float)period),period);
				h_err(ioctl(dev,BNS_SET_PERIODS,period));
				break;	
                        }
				
		}	
	while(option!=QUIT);
	printf("Setting slm power off\n");
	h_err(ioctl(dev,BNS_POWER_OFF,n));
	printf("Done, exiting\n");
	}	
	
	
void h_err(int err){
	if(err!=0){
		printf("Got error %d\n",err);
		exit(1);
		}
	else{
		printf("Ok\n");
		}
	usleep(1000);
	}
