#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h> 
#include <stdio.h>
#include "../../driver/slm.h"

#define BOOL	int
#define TRUE	1
#define FALSE	0

#define TEST "test.slm"
#define IMAGE	"test_image/long_bridge.slm"
#define SLM "/dev/slm"

#define NFRAMES 1024;
void h_err(int);

#define ROW_SIZE (512)
#define IMAGE_LENGTH 512
#define SIZE 512*IMAGE_LENGTH
#define N 100000


main(int argc,char ** argv){

int index=1;
int* dummy=NULL;
char *filename;
BOOL got_filename=FALSE;
float period;
BOOL set_period=FALSE;
int frame=0;
BOOL power_on=TRUE;
BOOL verbose_mode=FALSE;

FILE	*fi;
int	dev;
unsigned char image[SIZE];
int pos,y;
int n=0,err;

/*	printf("Steve's upload1_bns program\n");*/

	if(argc==1){
		printf("Need some arguments!\n\n");
		printf("-f   -filename    : filename of image (should have .bns extension)\n");
		printf("-fr  -frame       : frame to upload image to (default=0)\n");
		printf("-t   -time_period : time period (not set by default)\n");
		printf("-n   -no_power    : do NOT turn on SLM power (not default)\n");
		printf("-v   -verbose     : confirm what's been done (not default)\n");
		exit(3);
		}

	while(index<argc){
		if(sc(argv[index],"-v")||sc(argv[index],"-verbose")){
			verbose_mode=TRUE;
			}
		if(sc(argv[index],"-n")||sc(argv[index],"-no_power")){
			power_on=FALSE;
			}
		if(sc(argv[index],"-f")||sc(argv[index],"-filename")){
                        filename=argv[++index];
			got_filename=TRUE;
			}
		if(sc(argv[index],"-t")||sc(argv[index],"-time_period")){
			sscanf(argv[++index],"%f",&period);
			set_period=TRUE;
			}
		if(sc(argv[index],"-fr")||sc(argv[index],"-frame")){
			sscanf(argv[++index],"%d",&frame);
			}
		index++;
		}

	if(got_filename==FALSE){
		printf("ERROR: you must provide a filename\n");
		exit(1);
		}

	fi=fopen(filename,"r");
	if(fi>0){
		if(verbose_mode==TRUE){
			printf("Uploading \"%s\" to frame %d",filename,frame);
			}
		fread(image,sizeof(unsigned char),SIZE,fi);	
		fclose(fi);
		}
	else{
		printf("ERROR: could not open file \"%s\"\n",filename);
		exit(2);
		}

	dev=open(SLM,O_RDWR);
	if(dev<0){
		printf("Failed to open %s\n",SLM);
		perror("slm_test:FTO:");
		exit(1);
		}

        if(power_on==TRUE){
		h_err(ioctl(dev,BNS_POWER_ON,dummy));
                }
        else if(verbose_mode==TRUE){
		printf(", power NOT being turned on");
		}

	if(set_period==TRUE){
		if(verbose_mode==TRUE){
			printf(", period: %fs",period);
			}
		h_err(ioctl(dev,BNS_SET_PERIODS,(int)(period/164.4e-6)));
		}

	if(verbose_mode==TRUE){
		printf("\n");
		}

	h_err(ioctl(dev,BNS_ENABLE_WRITE,dummy));

	h_err(ioctl(dev,BNS_UPLOAD_IMAGE,frame));
	err=ROW_SIZE;
	for(pos=0,y=0;y<512;y++,pos+=IMAGE_LENGTH){ 
		err=write(dev,image+pos,ROW_SIZE);
		if(err!=ROW_SIZE){
			printf("WRITE ERROR (writes enabled?)\n",err);
			break;
			}
		}

	h_err(ioctl(dev,BNS_DISABLE_WRITE,dummy));
	h_err(ioctl(dev,BNS_DISPLAY_IMAGE,frame));
	}	
	
	
void h_err(int err){
	if(err!=0){
		printf("Got error %d\n",err);
		exit(1);
		}
	usleep(1000);
	}


BOOL sc(char *con,char *var){
	if(strcmp(con,var)==0){
		return TRUE;
		}
	return FALSE;
        }
